%ifndef __AF_FPU__
%define __AF_FPU__
	
%include "consts.asm"
%include "output.asm"


;
;     Fosforova FPU ASM knihovna
;
;     (c) by Aaka Fosfor, 2007
;     jabber: fosfor@jabber.cz
;     icq: 96560956
;     email: fosfor.software@seznam.cz
;
;     posledni uprava: 5.5.2007
;


; ----------------------------------------------------------------------
;           obsah knihovny
; ----------------------------------------------------------------------

; (void) fpu_print (st0 cislo)
;	vypise desetine cislo ulozene na vrcholu FPU a necha ho tam
; (void) fpu_printf (st0 cislo, cx pocet)
;	vypise desetine cislo ulozene na vrcholu FPU a necha ho tam
;	(vypise 2*pocet desetinych mist - povoleny rozsah <1;8> )
; (st0) fpu_factor (st0 cislo)
;	vypocte faktorial
; (st0) fpu_x_up_y (st1 x, st0 y)
;	vypocte celociselnou mocninu x^y

; makra
;	fpop
;		udela pop na FPU (a zmeni flagy)


; ----------------------------------------------------------------------
;           vnitrni promenne
; ----------------------------------------------------------------------

fpu_cw:		dw 0
fpu_znamenko:	db  0
fpu_exponent:	dw 0
fpu_mantisa:	times 10 db 0


; ----------------------------------------------------------------------
;           makra
; ----------------------------------------------------------------------

%macro fpop 0
	fcomp st1
%endmacro


; ----------------------------------------------------------------------
;           vlastni funkce
; ----------------------------------------------------------------------

; vypise desetine cislo ulozene na vrcholu FPU a necha ho tam
; vstup: st0
fpu_print:
	push cx
	mov cx, 8
	call fpu_printf
	pop cx
	ret


; vypise desetine cislo ulozene na vrcholu FPU a necha ho tam
;	(vypise 2*cx desetinych mist - povoleny rozsah <1;8> )
; vstup: st0
fpu_printf:
	push eax
	push cx
	push dx
	push si

	fldz
	fcomip st0, st1
	jne fpu_printf_nonz
	printc '0'
	jmp fpu_printf_end
fpu_printf_nonz:

	fwait
	; zaloha puvodniho cisla
	fld st0
	; zjistime, zda je X zaporne cislo
	fldz
	fcomip st0, st1
	mov byte [fpu_znamenko], '+'
	jb fpu_printf_positive
	; pokud ano, udelame X = abs(X)
	mov byte [fpu_znamenko], '-'
	fabs
fpu_printf_positive:

	; potrebujeme zjistit posunuti desetine carky
	; exp = log2(X)/log2(10) = log10(x)
	fld1
	fld st1
	fyl2x
	fldl2t
	fdivp st1, st0

	; zaokrouhlovat budeme smerem k -oo
	fstcw word [fpu_cw]
	and word [fpu_cw], 1111011111111111b
	or word [fpu_cw], 0000010000000000b
	fldcw word [fpu_cw]
	fwait

	; exp = round(exp)
	frndint

	; ulozime a jako exponent
	fist word [fpu_exponent]

	; potrebujeme posunout des. carku v X tak, aby bylo cele cislo reprezentovany jako int
	; (0.0032 => 32, 321.4 => 3214) - s tim pak pocitame, pri vypisu stejne zobrazime
	; des. carku za prvni cifrou
	; a = (exp - 16)       - o kolik budeme posouvat des. carku
	fld qword [c_16]
	fsubp st1, st0

	; potrebujeme: k = 10^a
	; a = a*log2(10)
	fld qword [c_10]
	fyl2x
	
	; a = 2^int(a)
	fld1
	fscale

	; b = a - int(a)
	fxch
	fld st0
	; tady zaokrouhlujeme smerem k nule (tj fce int(x) v minulych dvou komentarich)	
	fstcw word [fpu_cw]
	or word [fpu_cw], 0000110000000000b
	fldcw word [fpu_cw]
	frndint
	fsubp st1, st0

	; b = 2^b     - tady muze byt b jen <-1;1>
	f2xm1
	fld1
	faddp st1, st0
	fmulp st1, st0

	fdivp st1, st0
	fbstp [fpu_mantisa]

	fwait
	; vytisknout BCD
	printc [fpu_znamenko]
	
	fwait
	; kontrola spravnosti cx <1;8>
	cmp cx, 0
	jne fpu_printf_cx1
	mov cx, 8
fpu_printf_cx1:
	cmp cx, 9
	jb fpu_printf_cx2
	mov cx, 8
fpu_printf_cx2:

	mov si, fpu_mantisa
	add si, 8
	xor dx, dx
	mov al, [si]
	mov dl, al
	and dl, 15
	add dl, 48
	call print_char
	printc '.'
	dec si
fpu_printf_print:
	mov al, [si]
	mov dl, al
	shr dl, 4
	add dl, 48
	call print_char
	mov dl, al
	and dl, 15
	add dl, 48
	call print_char
	dec si
	loop fpu_printf_print
	printc 'e'
	xor eax, eax
	mov ax, [fpu_exponent]
	call print_signed

fpu_printf_end:
	pop si
	pop dx
	pop cx
	pop eax
	ret


; vypocte faktorial
; vstup: st0
; vystup: st0
fpu_factor:
	fld st0
fpu_factor_cyklus:
	fld1
	fcomip st0, st1
	jae fpu_factor_end
	fld1
	fsubp st1, st0
	fmul st1, st0
	jmp fpu_factor_cyklus
fpu_factor_end:
	; odemereme pomocne cislo z vrcholu
	fcomip st0, st1
	ret


; vypocte celociselnou mocninu: st0 = st0 ^ st1
; vstup: st0, st1
; vystup: st0
fpu_x_up_y:
	; tady zaokrouhlujeme smerem k nule
	fstcw word [fpu_cw]
	or word [fpu_cw], 0000110000000000b
	fldcw word [fpu_cw]

	; y = int(y)
	frndint
	; if y == 0, return 1
	fldz
	fcomip st0, st1
	je fpu_x_up_y_zero

	; vememe x, pocatek vysledku
	fld st1

fpu_x_up_y_cyklus:
	fxch
	; y = y - 1
	fld1
	fsubp st1, st0
	; if y == 0, return vysledek
	fldz
	fcomip st0, st1
	je fpu_x_up_y_end
	fxch
	fld st2
	fmulp st1, st0
	jmp fpu_x_up_y_cyklus


fpu_x_up_y_end:
	; vybereme zasobnik a vratim vysledek (ten je ale zatim navrchu;)
	fcomip st0, st1
	fxch
	fcomip st0, st1
	ret


fpu_x_up_y_zero:
	; vybereme zasobnik
	fcomip st0, st1
	fcomip st0, st1
	; vratime 1
	fld1	
	ret


%endif ;__AF_FPU__