%ifndef __AF_MMX__
%define __AF_MMX__

%include "output.asm"
%include "system.asm"

;
;     Fosforova ASM knihovna pro jine instrukcni sady
;
;     (c) by Aaka Fosfor, 2007
;     jabber: fosfor@jabber.cz
;     icq: 96560956
;     email: fosfor.software@seznam.cz
;
;     posledni uprava: 6.5.2007
;


; ----------------------------------------------------------------------
;           obsah knihovny
; ----------------------------------------------------------------------


; (void) cpuid_check (void)
;	overi, zda je CPU schopne vykonat CPUID, pokud ne, skonci program
; (void) mmx_check (void)
;	overi, zda je CPU schopne vykonavat MMX instrukce, pokud ne, skonci program
; (void) sse_check (void)
;	overi, zda je CPU schopne vykonavat SSE instrukce, pokud ne, skonci program
; (void) sse2_check (void)
;	overi, zda je CPU schopne vykonavat SSE2 instrukce, pokud ne, skonci program
; (void) print_mmx (void)
;	vytiskne cislo v prvnim MMX registru (mm0)


; ----------------------------------------------------------------------
;           vnitrni promenne
; ----------------------------------------------------------------------


mmx_no_cpuid:
	db "CPU doesn't suport CPUID instruction. Can't continue.",13,10,'$'
mmx_no_mmx:
	db "CPU doesn't suport MMX feature. Can't continue.",13,10,'$'
mmx_no_sse:
	db "CPU doesn't suport SSE feature. Can't continue.",13,10,'$'
mmx_no_sse2:
	db "CPU doesn't suport SSE2 feature. Can't continue.",13,10,'$'
mmx_tmp:
	times 8 db 0
mmx_mmx0:
	db "MMX0: $"


; ----------------------------------------------------------------------
;           vlastni funkce
; ----------------------------------------------------------------------

; overi, zda je CPU schopne vykonat CPUID, pokud ne, skonci program
cpuid_check:
	push eax
	push ebx

	; can do CPUID ?
	pushfd ; save EFLAGS
	pop eax ; store EFLAGS in EAX
	mov ebx, eax ; save in EBX for later testing
	xor eax, 00200000h ; toggle bit 21
	push eax ; put to stack
	popfd ; save changed EAX to EFLAGS
	pushfd ; push EFLAGS to TOS
	pop eax ; store EFLAGS in EAX
	cmp eax, ebx ; see if bit 21 has changed
	jnz cpuid_check_ok ; if no change, no CPUID

	print mmx_no_cpuid
	exit 1

cpuid_check_ok:
	pop ebx
	pop eax
	ret


; overi, zda je CPU schopne vykonavat MMX instrukce, pokud ne, skonci program
mmx_check:
	push eax
	push ebx
	push ecx
	push edx ; nutno ulozit, cpuid meni vse

	call cpuid_check

	mov eax, 1 ; pozadavek na vzhled ekstendyt flagu
	cpuid
	test edx, 1 << 23 ; bit 23 - MMX
	jnz mmx_check_found

	print mmx_no_mmx
	exit 1

mmx_check_found:
	pop edx
	pop ecx
	pop ebx
	pop eax
	ret

; overi, zda je CPU schopne vykonavat SSE instrukce, pokud ne, skonci program
sse_check:
	push eax
	push ebx
	push ecx
	push edx ; nutno ulozit, cpuid meni vse

	call cpuid_check

	mov eax, 1 ; pozadavek na vzhled ekstendyt flagu
	cpuid
	test edx, 1 << 25 ; bit 25 - SSE
	jnz sse_check_found

	print mmx_no_sse
	exit 1

sse_check_found:
	pop edx
	pop ecx
	pop ebx
	pop eax
	ret

; overi, zda je CPU schopne vykonavat SSE2 instrukce, pokud ne, skonci program
sse2_check:
	push eax
	push ebx
	push ecx
	push edx ; nutno ulozit, cpuid meni vse

	call cpuid_check

	mov eax, 1 ; pozadavek na vzhled ekstendyt flagu
	cpuid
	test edx, 1 << 26 ; bit 26 - SSE2
	jnz sse2_check_found

	print mmx_no_sse2
	exit 1

sse2_check_found:
	pop edx
	pop ecx
	pop ebx
	pop eax
	ret

; vytiskne cislo v prvnim MMX registru (mm0)
print_mmx:
	push cx
	push ax
	push bx

	printp mmx_mmx0
	movq [cs:mmx_tmp], mm0
	mov cx, 4
	xor bx, bx
print_mmx_loop:
	mov ax, [cs:mmx_tmp+bx]
	inc bx
	inc bx
	call print_hex
	loop print_mmx_loop

	pop bx
	pop ax
	pop cx
	ret

%endif ;__AF_MMX__