%ifndef __AF_OUTPUT__
%define __AF_OUTPUT__

%include "math.asm"


;
;     Fosforova vystupni ASM knihovna
;
;     (c) by Aaka Fosfor, 2007
;     jabber: fosfor@jabber.cz
;     icq: 96560956
;     email: fosfor.software@seznam.cz
;
;     posledni uprava: 8.5.2007
;


; ----------------------------------------------------------------------
;           obsah knihovny
; ----------------------------------------------------------------------


; (void) print_at (al - radka, ah - sloupec, bh - barva, ds:dx - retezec ASCIIZ)
;	na pozici al;ah vytiskne retezec z ds:dx barvou bh (jen v textovem modu a modu 13)
; (void) print_signed32 (eax cislo)
;	vypise desitkove znamenkove cislo ulozene v eax
; (void) print_signed (ax cislo)
;	vypise desitkove znamenkove cislo ulozene v ax
; (void) print_hex32 (eax cislo)
;	vypise hexa cislo ulozeny v eax
; (void) print_hex_wz (ax cislo)
;	vypise hexa cislo ulozeny v ax (bez pocatecni nuly)
; (void) print_hex (ax cislo)
;	vypise hexa cislo ulozeny v ax (vcetne pocatecni nuly)
; (void) print_number32 (eax cislo)
;	vypise desitkove cislo ulozene v eax
; (void) print_number (ax cislo)
;	vypise desitkove cislo ulozene v ax
; (void) print_string (dx adresa)
;	vypise retezec ze segmentu programu
; (void) print_char (dl ascii_znaku)
;	vypise jeden znak

; konstanty
;	endl - konec radku (DOS retezec)

; makra:
;	printc <ascii znaku>
;		vytiskne jeden znak, nezmeni nic
;	print <adresa retezce v CS>
;		alias pro printp
;	printp <adresa retezce v CS>
;		vytiskne retezec a nezmeni dx
;	printn32 <adresa 32-bit cisla>
;		vytiskne 32-bit cislo (meni dx)
;	printn32p <adresa 32-bit cisla>
;		vytiskne 32-bit cislo a nezmeni dx


; ----------------------------------------------------------------------
;           konstanty
; ----------------------------------------------------------------------


endl:	db 13,10,'$'


; ----------------------------------------------------------------------
;           makra nad funkcemi
; ----------------------------------------------------------------------

%macro printc 1
	push dx
	mov dx, %1
	call print_char
	pop dx
%endmacro

%macro print 1
	printp %1
%endmacro

%macro printp 1
	push dx
	mov dx, %1
	call print_string
	pop dx
%endmacro 

%macro printn32 1
	mov eax, [%1]
	call print_number32
%endmacro

%macro printn32p 1
	push eax
	mov eax, [%1]
	call print_number32
	pop eax
%endmacro


; ----------------------------------------------------------------------
;           vlastni funkce
; ----------------------------------------------------------------------


; (void) print_at (al - radka, ah - sloupec, bh - barva, ds:si - retezec ASCIIZ)
;	na pozici al;ah vytiskne retezec z ds:dx barvou bl (jen v textovem modu a modu 13)
print_at:
	push es
	push di
	push bx
	push si
	push dx

	xor dx, dx
	mov dl, al
	push dx
	shl dx, 2
	pop di
	add dx, di ; dx = 5 * al
	shl dx, 4 ; dx = 80 * al
	add dl, ah
	adc dh, 0 ; dx = 80 * al + ah
	shl dx, 1 ; *2 - kazdy znak zabira 2 byty (ascii, barva)
	
	push word 0b800h
	pop es
	mov di, dx
print_at_cyklus:
	mov bl, [ds:si]
	cmp bl, 0
	je print_at_konec
	inc si
	mov [es:di], bx
	inc di
	inc di
	jmp short print_at_cyklus

print_at_konec:
	pop dx
	pop si
	pop bx
	pop di
	pop es
	ret


; print_signed32
; vypise desitkove znamenkove cislo ulozene v eax
; vstup: eax - cislo; vystup: nic
print_signed32:
	push eax
	push ebx

	push eax

	rol eax, 1
	jnc print_signet32_print

	mov dl, '-'
	mov ah, 2
	int 21h
	pop ebx
	mov eax, 0FFFFFFFFh
	sub eax, ebx
	inc eax
	push eax

print_signet32_print:
	pop eax
	call print_number32

	pop ebx
	pop eax
	ret


; print_signed
; vypise desitkove znamenkove cislo ulozene v ax
; vstup: ax - cislo; vystup: nic
print_signed:
	push ax
	push bx

	push ax

	rol ax, 1
	jnc print_signet_print

	mov dl, '-'
	mov ah, 2
	int 21h
	pop bx
	mov ax, 0FFFFh
	sub ax, bx
	inc ax
	push ax

print_signet_print:
	pop ax
	call print_number32

	pop bx
	pop ax
	ret


; print_hex32
; vypise hexa cislo ulozeny v eax
; vstup: eax - cislo; vystup: nic
print_hex32:
	push eax
	push edx
	push cx

	xor cx, cx

print_hex32_cyklus:
	mov edx, eax
	and edx, 15
	push dx
	inc cx
	shr eax, 4
	cmp eax, 0
	jne print_hex32_cyklus


print_hex32_print:
	pop dx
	add dl, 48
	cmp dl, 58
	jl print_hex32_print_num
	add dl, 7
print_hex32_print_num:
	mov ah, 2
	int 21h
	loop print_hex32_print

	pop cx
	pop edx
	pop eax

	ret


; print_hex_wz
; vypise hexa cislo ulozeny v ax (bez pocatecni nuly)
; vstup: ax - cislo; vystup: nic
print_hex_wz:
	push ax
	push dx
	push cx

	xor cx, cx

print_hex_wz_cyklus:
	mov dx, ax
	and dx, 15
	push dx
	inc cx
	shr ax, 4
	cmp ax, 0
	jne print_hex_wz_cyklus


print_hex_wz_print:
	pop dx
	add dl, 48
	cmp dl, 58
	jl print_hex_wz_print_num
	add dl, 7
print_hex_wz_print_num:
	mov ah, 2
	int 21h
	loop print_hex_wz_print

	pop cx
	pop dx
	pop ax

	ret


; print_hex
; vypise hexa cislo ulozeny v ax (vcetne pocatecni nuly)
; vstup: ax - cislo; vystup: nic
print_hex:
	push ax
	push dx
	push cx

	mov cx, 4
print_hex_cyklus:
	mov dx, ax
	and dx, 15
	push dx
	shr ax, 4
	loop print_hex_cyklus

	mov cx, 4
print_hex_print:
	pop dx
	add dl, 48
	cmp dl, 58
	jl print_hex_print_num
	add dl, 7
print_hex_print_num:
	mov ah, 2
	int 21h
	loop print_hex_print

	pop cx
	pop dx
	pop ax

	ret


; print_number32
; vypise desitkove cislo ulozene v eax
; vstup: eax - cislo; vystup: nic
print_number32:
	push eax
	push ebx
	push edx
	push cx

	xor cx, cx

print_number32_cyklus:
	mov ebx, 10
	call deleni32

	push dx
	inc cx

	cmp eax, 0
	jne print_number32_cyklus

print_number32_print:
	pop dx
	add dl, 48
	mov ah, 2
	int 21h
	loop print_number32_print
	
print_number32_konec:
	pop cx
	pop edx
	pop ebx
	pop eax

	ret

; print_number
; vypise desitkove cislo ulozene v ax
; vstup: ax - cislo; vystup: nic
print_number:
	push ax
	push bx
	push dx
	push cx
	pushf

	xor cx, cx

print_number_cyklus:
	xor dx, dx
	mov bx, 10
	div bx

	push dx
	inc cx

	cmp ax, 0
	jne print_number_cyklus

print_number_print:
	pop dx
	add dl, 48
	mov ah, 2
	int 21h
	loop print_number_print
	
print_number_konec:
	popf
	pop cx
	pop dx
	pop bx
	pop ax

	ret

; print_string
; vypise retezec ze segmentu programu
; vstup: dx - adresa retezce
print_string:
	push ax
	push ds
	pushf

	push cs
	pop ds
	xor ax, ax
	mov ah, 09h
	int 21h

	popf
	pop ds
	pop ax
	ret

; print_char
; vypise jeden znak
; vstup: dl - ascii znaku
print_char:
	push ax

	mov ah, 2
	int 21h

	pop ax
	ret

%endif ;__AF_OUTPUT__